use crate::state::State;
use crate::{Gsa, Rgb, Sprite, MAX_SPRITES};
use gilrs::Gilrs;
use skunk2d::Image8;

/// Creates main function, includes gfx.gif, and calls run
///
/// Pass the following two functions to it:
/// - fn init(gsa: &mut Gsa) -> TGame
/// - fn update(game: &mut Game, gsa: &mut Gsa)
///
/// TGame can be any type, usually a struct that contains your game's state
#[macro_export]
macro_rules! run {
    ($init: ident, $update: ident) => {
        fn main() {
            run($init, $update, include_bytes!("gfx.gif"));
        }
    };
}

/// This is called by [run!]
pub fn run<TGame: 'static>(
    init_fn: fn(gsa: &mut Gsa) -> TGame,
    update_fn: fn(game: &mut TGame, gsa: &mut Gsa),
    image_data: &[u8],
) {
    let tileset = Image8::load_data(image_data);
    let pal = Image8::load_data_palette(image_data);
    let mut palette = [Rgb { r: 0, g: 0, b: 0 }; 256];

    for i in 0..pal.len() / 3 {
        palette[i] = Rgb {
            r: pal[i * 3 + 2],
            g: pal[i * 3 + 1],
            b: pal[i * 3],
        };
    }

    let mut gsa = Gsa {
        sprites: [Sprite::default(); MAX_SPRITES],
        maps: Default::default(),
        palette,
        font: 0x1010,
        input: Default::default(),
    };
    let game = init_fn(&mut gsa);
    let state = State::<TGame> {
        gsa,
        game,
        tileset,
        update_fn,
        first: true,
        gilrs: Gilrs::new().unwrap(),
    };
    skunk2d::run_with::<Image8, State<TGame>>(304, 176, 60, state);
}
