use crate::buttons::Buttons;
use crate::{Gsa, TILEMAP_MAX_SIZE};
use gilrs::{Button, EventType, Gilrs};
use glam::IVec2;
use skunk2d::{Event, IRect, Image8, WindowState};
use std::rc::Rc;

pub struct State<TGame> {
    pub(crate) gsa: Gsa,
    pub(crate) game: TGame,
    pub(crate) update_fn: fn(game: &mut TGame, gsa: &mut Gsa),
    pub(crate) tileset: Rc<Image8>,
    pub(crate) first: bool,
    pub(crate) gilrs: Gilrs,
}

impl<TGame> skunk2d::Game<Image8> for State<TGame> {
    fn update(&mut self, window_state: &mut WindowState) {
        if self.first {
            window_state.toggle_fullscreen();
            self.first = false;
        }
        let mut new_buttons = self.gsa.input.down;
        while let Some(event) = self.gilrs.next_event() {
            match event {
                gilrs::Event {
                    event: EventType::ButtonPressed(button, _),
                    ..
                } => match button {
                    Button::South => new_buttons.face_down = true,
                    Button::East => new_buttons.face_left = true,
                    Button::North => new_buttons.face_up = true,
                    Button::West => new_buttons.face_right = true,
                    Button::LeftTrigger | Button::LeftTrigger2 => new_buttons.l = true,
                    Button::RightTrigger | Button::RightTrigger2 => new_buttons.r = true,
                    Button::Select => new_buttons.select = true,
                    Button::Start => new_buttons.start = true,
                    Button::DPadUp => new_buttons.dpad_up = true,
                    Button::DPadDown => new_buttons.dpad_down = true,
                    Button::DPadLeft => new_buttons.dpad_left = true,
                    Button::DPadRight => new_buttons.dpad_right = true,
                    _ => {}
                },
                gilrs::Event {
                    event: EventType::ButtonReleased(button, _),
                    ..
                } => match button {
                    Button::South => new_buttons.face_down = false,
                    Button::East => new_buttons.face_left = false,
                    Button::North => new_buttons.face_up = false,
                    Button::West => new_buttons.face_right = false,
                    Button::LeftTrigger | Button::LeftTrigger2 => new_buttons.l = false,
                    Button::RightTrigger | Button::RightTrigger2 => new_buttons.r = false,
                    Button::Select => new_buttons.select = false,
                    Button::Start => new_buttons.start = false,
                    Button::DPadUp => new_buttons.dpad_up = false,
                    Button::DPadDown => new_buttons.dpad_down = false,
                    Button::DPadLeft => new_buttons.dpad_left = false,
                    Button::DPadRight => new_buttons.dpad_right = false,
                    _ => {}
                },
                _ => (),
            }
        }
        self.gsa.input.pressed = Buttons::pressed(&self.gsa.input.down, &new_buttons);
        self.gsa.input.down = new_buttons;
        self.gsa.input.dir = IVec2 {
            x: if self.gsa.input.down.dpad_left { -1 } else { 0 }
                + if self.gsa.input.down.dpad_right { 1 } else { 0 },
            y: if self.gsa.input.down.dpad_up { -1 } else { 0 }
                + if self.gsa.input.down.dpad_down { 1 } else { 0 },
        };

        //todo: don't if not updated... how check? <_<
        for i in 0..=255 {
            window_state.set_palette(
                i,
                self.gsa.palette[i as usize].r,
                self.gsa.palette[i as usize].g,
                self.gsa.palette[i as usize].b,
            );
        }
        (self.update_fn)(&mut self.game, &mut self.gsa);
    }

    fn on_event(&mut self, _window_state: &mut WindowState, _event: Event) {}

    fn draw(&self, target: &mut Image8) {
        target.clear();

        for map in &self.gsa.maps {
            let tcmult = if map.half_tile { 2 } else { 1 };
            let tilesize = if map.half_tile { 8 } else { 16 };
            let mut startx = map.scroll.x / tilesize;
            let mut starty = map.scroll.y / tilesize;
            let endx = (TILEMAP_MAX_SIZE as i32).min(startx + 20 * tcmult);
            let endy = (TILEMAP_MAX_SIZE as i32).min(starty + 12 * tcmult);
            startx = 0.max(startx);
            starty = 0.max(starty);
            for x in startx..endx {
                for y in starty..endy {
                    let tile = map.tiles[x as usize][y as usize];
                    if tile > 0 {
                        let ty = tile / 0x100;
                        let tx = tile % 0x100;
                        target.draw_image_partial(
                            IVec2 {
                                x: x * tilesize - map.scroll.x,
                                y: y * tilesize - map.scroll.y,
                            },
                            &self.tileset,
                            IRect {
                                pos: IVec2 {
                                    x: tx as i32 * tilesize,
                                    y: ty as i32 * tilesize,
                                },
                                size: IVec2 {
                                    x: tilesize,
                                    y: tilesize,
                                },
                            },
                        )
                    }
                }
            }
        }

        for sprite in &self.gsa.sprites {
            if sprite.tile > 0 {
                let ty = sprite.tile / 0x100;
                let tx = sprite.tile % 0x100;
                target.draw_image_partial(
                    sprite.pos,
                    &self.tileset,
                    IRect {
                        pos: IVec2 {
                            x: tx as i32 * 16,
                            y: ty as i32 * 16,
                        },
                        size: IVec2 { x: 16, y: 16 },
                    },
                );
            }
        }
    }
}
